==> azureml_train_automl-1.0.17.1-py3-none-any.whl: azureml/train/automl/_vendor/automl/client/core/common/featurization/categorical/cat_imputer.py <==
# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Imputer for missing values in Categorical data."""

import numpy as np
import pandas as pd
from sklearn.base import BaseEstimator, TransformerMixin

from automl.client.core.common.logging_utilities import function_debug_log_wrapped
from automl.client.core.common.featurization.logger import TransformerLogger


class CatImputer(BaseEstimator, TransformerMixin, TransformerLogger):
    """Impute missing values for categorical data by the most frequent category."""

    def __init__(self, copy: bool = True, logger: TransformerLogger = None):
        """
        Construct the CatImputer.

        :param copy: Create copy of the categorical column.
        :param logger: Logger to be injected to usage in this class.
        :return:
        """
        self._missing_vals = [np.nan]
        self._copy = copy
        self._init_logger(logger)

    def _get_mask(self, x):
        """
        Get missing values mask.

        :param x: Input array.
        :return: Mask with missing values.
        """
        mask = np.zeros(x.shape, dtype=bool)
        x_object = None
        for val in self._missing_vals:
            if val is None or (isinstance(val, float) and np.isnan(val)):
                mask = mask | pd.isnull(x)
            else:
                x_object = x.astype(
                    np.object) if x_object is None else x_object
                mask = mask | (x_object == val)

        return mask

    def fit(self, x, y=None):
        """
        Transform the data to mark the missing values and identify the most frequent category.

        :param x: The data to transform.
        :type x: numpy.ndarray or pandas.series
        :param y: Target values.
        :type y: numpy.ndarray
        :return: The instance object: self.
        """
        non_na = x.dropna()
        if non_na.empty:
            self._fill = str(np.nan)
            return self

        series_name = x.name if x.name is not None else 0

        mode = non_na.to_frame().groupby(series_name)[
            series_name].agg("count").idxmax()
        self._fill = mode

        return self

    @function_debug_log_wrapped
    def transform(self, x):
        """
        Transform data x by adding the missing values with the most frequent categories.

        Must call fit() before calling transform()

        :param x: The data to transform.
        :type x: numpy.ndarray or pandas.series
        :return: The transformed data.
        """
        if self._fill is None:
            raise ValueError("CatImputer fit not called")

        mask = self._get_mask(x)
        if self._copy:
            x = x.copy()
        x[mask] = self._fill

        return x.values

==> azureml_train_automl-1.0.18-py3-none-any.whl: azureml/train/automl/_vendor/automl/client/core/common/featurization/categorical/cat_imputer.py <==
# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Imputer for missing values in Categorical data."""
from typing import Optional, Union
from logging import Logger
import numpy as np
import pandas as pd
from sklearn.base import BaseEstimator, TransformerMixin

from automl.client.core.common.logging_utilities import function_debug_log_wrapped
from automl.client.core.common.featurization.logger import TransformerLogger


class CatImputer(BaseEstimator, TransformerMixin, TransformerLogger):
    """Impute missing values for categorical data by the most frequent category."""

    def __init__(self, copy: bool = True, logger: Optional[Union[Logger, TransformerLogger]] = None):
        """
        Construct the CatImputer.

        :param copy: Create copy of the categorical column.
        :param logger: Logger to be injected to usage in this class.
        :return:
        """
        self._missing_vals = [np.nan]
        self._copy = copy
        self._init_logger(logger)

    def _get_mask(self, x):
        """
        Get missing values mask.

        :param x: Input array.
        :return: Mask with missing values.
        """
        mask = np.zeros(x.shape, dtype=bool)
        x_object = None
        for val in self._missing_vals:
            if val is None or (isinstance(val, float) and np.isnan(val)):
                mask = mask | pd.isnull(x)
            else:
                x_object = x.astype(
                    np.object) if x_object is None else x_object
                mask = mask | (x_object == val)

        return mask

    def fit(self, x, y=None):
        """
        Transform the data to mark the missing values and identify the most frequent category.

        :param x: The data to transform.
        :type x: numpy.ndarray or pandas.series
        :param y: Target values.
        :type y: numpy.ndarray
        :return: The instance object: self.
        """
        non_na = x.dropna()
        if non_na.empty:
            self._fill = str(np.nan)
            return self

        series_name = x.name if x.name is not None else 0

        mode = non_na.to_frame().groupby(series_name)[
            series_name].agg("count").idxmax()
        self._fill = mode

        return self

    @function_debug_log_wrapped
    def transform(self, x):
        """
        Transform data x by adding the missing values with the most frequent categories.

        Must call fit() before calling transform()

        :param x: The data to transform.
        :type x: numpy.ndarray or pandas.series
        :return: The transformed data.
        """
        if self._fill is None:
            raise ValueError("CatImputer fit not called")

        mask = self._get_mask(x)
        if self._copy:
            x = x.copy()
        x[mask] = self._fill

        return x.values
